<?php
/* Copyright (C) 2001-2005 Rodolphe Quiedeville <rodolphe@quiedeville.org>
 * Copyright (C) 2004-2015 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2005-2012 Regis Houssin        <regis.houssin@inodbox.com>
 * Copyright (C) 2015      Jean-François Ferry	<jfefe@aternatik.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

/**
 *	\file       importupdateproduct/importupdateproductindex.php
 *	\ingroup    importupdateproduct
 *	\brief      Home page of importupdateproduct top menu
 */

// Load Dolibarr environment
$res = 0;
// Try main.inc.php into web root known defined into CONTEXT_DOCUMENT_ROOT (not always defined)
if (!$res && !empty($_SERVER["CONTEXT_DOCUMENT_ROOT"])) $res = @include $_SERVER["CONTEXT_DOCUMENT_ROOT"]."/main.inc.php";
// Try main.inc.php into web root detected using web root calculated from SCRIPT_FILENAME
$tmp = empty($_SERVER['SCRIPT_FILENAME']) ? '' : $_SERVER['SCRIPT_FILENAME']; $tmp2 = realpath(__FILE__); $i = strlen($tmp) - 1; $j = strlen($tmp2) - 1;
while ($i > 0 && $j > 0 && isset($tmp[$i]) && isset($tmp2[$j]) && $tmp[$i] == $tmp2[$j]) { $i--; $j--; }
if (!$res && $i > 0 && file_exists(substr($tmp, 0, ($i + 1))."/main.inc.php")) $res = @include substr($tmp, 0, ($i + 1))."/main.inc.php";
if (!$res && $i > 0 && file_exists(dirname(substr($tmp, 0, ($i + 1)))."/main.inc.php")) $res = @include dirname(substr($tmp, 0, ($i + 1)))."/main.inc.php";
// Try main.inc.php using relative path
if (!$res && file_exists("../main.inc.php")) $res = @include "../main.inc.php";
if (!$res && file_exists("../../main.inc.php")) $res = @include "../../main.inc.php";
if (!$res && file_exists("../../../main.inc.php")) $res = @include "../../../main.inc.php";
if (!$res) die("Include of main fails");

require_once DOL_DOCUMENT_ROOT . '/core/class/html.formfile.class.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/html.formother.class.php';
require_once DOL_DOCUMENT_ROOT . '/core/lib/files.lib.php';
require_once DOL_DOCUMENT_ROOT . '/core/lib/date.lib.php';
dol_include_once('/importupdateproduct/class/importupdateproduct.class.php');

// Load translation files required by the page
$langs->loadLangs(array("importupdateproduct@importupdateproduct"));

$id=GETPOST('id', 'int');
$action=GETPOST('action', 'alpha');
$confirm=GETPOST('confirm', 'alpha');


// Security check
if (! $user->rights->importupdateproduct->read) accessforbidden();
$socid = GETPOST('socid', 'int');
if (isset($user->socid) && $user->socid > 0)
{
	$action = '';
	$socid = $user->socid;
}

$max = 5;
$now = dol_now();

$form = new Form($db);
$formfile = new FormFile($db);
$formother = new FormOther($db);
/*
 * Actions
 */

// None

if ($action == 'delete')
{
	$langs->load("companies");	// Need for string DeleteFile+ConfirmDeleteFiles
	$ret = $form->form_confirm(
		$_SERVER["PHP_SELF"] . '?urlfile=' . urlencode(GETPOST("urlfile")) . '&linkid=' . GETPOST('linkid', 'int') . (empty($param)?'':$param),
		$langs->trans('DeleteFile'),
		$langs->trans('ConfirmDeleteFile'),
		'confirm_deletefile',
		'',
		0,
		1
	);
	if ($ret == 'html') print '<br>';
}
if($action == "confirm_deletefile" && $confirm=="yes"){
	$file = $upload_dir . "/" . GETPOST('urlfile');	// Do not use urldecode here ($_GET and $_POST are already decoded by PHP).
	$ret=dol_delete_file($file);
	header('Location:' . $_SERVER["PHP_SELF"]);
}
/*
 * View
 */
$form = new Form($db);
$formfile = new FormFile($db);


$title = $langs->trans("Importupdateproduct");

$upload_dir = $conf->importupdateproduct->dir_output;
$filearray=dol_dir_list($upload_dir, "files", 0, '', '(\.meta|_preview\.png)$', $sortfield, (strtolower($sortorder)=='desc'?SORT_DESC:SORT_ASC), 1);


$langs->load("link");
if (empty($relativepathwithnofile)) $relativepathwithnofile='';
if (empty($permtoedit)) $permtoedit=-1;

if(GETPOST('import')==1 && !empty(GETPOST('fimport'))){
	$excel = new Importupdateproduct($db);
	$excel->sheetprocess(GETPOST('fimport'));
	header('Location: '.DOL_URL_ROOT.'/product/index.php?leftmenu=product&type=0');
}
llxHeader('', $title);

/*
 * Confirm form to delete
 */
$permissiontoadd = 1;
include_once DOL_DOCUMENT_ROOT . '/core/actions_linkedfiles.inc.php';
$afile = GETPOST('sendit');
//echo "<pre>" . print_r($_POST, 1) . "</pre>";
if(!empty($afile)){
	$file = $_FILES['userfile'];
	if(!file_exists($upload_dir)){
		dol_mkdir($upload_dir);
	}

	$dest_file = $upload_dir.dol_sanitizeFileName($file['name'][0]);

	$result = dol_move_uploaded_file($file['tmp_name'][0], $dest_file, 1, 0, 0);

	//
	if(file_exists($dest_file)){
		$excel = new Importupdateproduct($db);
		$excel->sheettest($dest_file);
		header('Location: '.$_SERVER['PHP_SELF']);
	}
}
if ($action == 'delete')
{
	$langs->load("companies");	// Need for string DeleteFile+ConfirmDeleteFiles
	$ret = $form->form_confirm(
		$_SERVER["PHP_SELF"] . '?urlfile=' . urlencode(GETPOST("urlfile")) . '&linkid=' . GETPOST('linkid', 'int') . (empty($param)?'':$param),
		$langs->trans('DeleteFile'),
		$langs->trans('ConfirmDeleteFile'),
		'confirm_deletefile',
		'',
		0,
		1
	);
	if ($ret == 'html') print '<br>';
}
if($action == "confirm_deletefile" && $confirm=="yes"){
	$file = $upload_dir . "/" . GETPOST('urlfile');	// Do not use urldecode here ($_GET and $_POST are already decoded by PHP).
	$ret=dol_delete_file($file);
	header('Location:' . $_SERVER["PHP_SELF"]);
}


print_titre("ImportProduct");
$formfile=new FormFile($db);

// We define var to enable the feature to add prefix of uploaded files
//$savingdocmask=dol_sanitizeFileName(1).'-__file__';


$modulepart = 'importupdateproduct';
$permission = $user->rights->importupdateproduct->write;
$permtoedit = $user->rights->importupdateproduct->read;
$param = '&id=' . 1;
// Show upload form (document and links)
$formfile->form_attach_new_file(
	$_SERVER["PHP_SELF"],
	'',
	0,
	0,
	$permission,
	$conf->browser->layout == 'phone' ? 40 : 60,
	'',
	'',
	1,
	$savingdocmask, 0, 'importupdateproduct'
);

echo '<a href="' . dol_buildpath('importupdateproduct/exemple.xlsx', 1) . '" class="button">Fichier Exemple</a>';

// List of document

$formfile->list_of_documents(
	$filearray,
	$object,
	$modulepart,
	$param,
	0,
	$relativepathwithnofile,		// relative path with no file. For example "moduledir/0/1"
	1,
	0,
	'',
	0,
	'',
	'',
	0,
	0,
	'',
	'date',
	'DESC'
);

print "<br>";

// End of page
llxFooter();
$db->close();
